<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2016 Amasty (https://www.amasty.com)
 * @package Mageplace_Gallery
 */

/**
 * Class Mageplace_Gallery_Block_Photo_Widget_List
 */
class Mageplace_Gallery_Block_Photo_Widget_List
    extends Mageplace_Gallery_Block_Photo_List_Abstract
    implements Mage_Widget_Block_Interface
{
    const DEFAULT_PHOTOS_PER_PAGE   = 4;
    const DEFAULT_PHOTOS_SORT_BY    = 'name';
    const DEFAULT_PHOTOS_SORT_DIR   = 'ASC';
    const DEFAULT_PHOTOS_SIZE       = '640x480';
    const DEFAULT_PHOTOS_THUMB_SIZE = '135x135';

    protected $_template = 'mpgallery/photo/widget/list.phtml';

    protected $_displayName        = false;
    protected $_displayDescription = false;
    protected $_displayUpdateDate  = false;
    protected $_displayShowLink    = false;

    public function isEnabled()
    {
        if (!$this->hasData('is_enabled')) {
            $this->setData('is_enabled', $this->getAlbumId() > 0);
        }

        return $this->_getData('is_enabled');
    }

    /**
     * @return Mageplace_Gallery_Model_Mysql4_Photo_Collection|null
     */
    public function getCollection()
    {
        if (!$this->hasData('collection')) {
            $this->setData('collection',
                Mage::getModel('mpgallery/photo')->getCollection()
                    ->addAlbumFilter($this->getAlbumId())
                    ->addIsActiveFilter()
                    ->addStoreFilter()
                    ->addCustomerGroupFilter()
            );
        }

        return $this->_getData('collection');
    }

    public function getPhotos()
    {
        return $this->getCollection();
    }

    public function getAlbumId()
    {
        if ($this->hasData('album_id')) {
            return (int)$this->getData('album_id');
        }

        return false;
    }

    public function getTitle()
    {
        if ($this->hasData('title')) {
            return $this->getData('title');
        }

        return $this->__('Photos');
    }


    public function getPhotoSize()
    {
        if ($this->hasData('photo_size')) {
            return $this->getData('photo_size');
        }

        return self::DEFAULT_PHOTOS_SIZE;
    }


    public function getPhotoThumbSize()
    {
        if ($this->hasData('photo_thumb_size')) {
            return $this->getData('photo_thumb_size');
        }

        return self::DEFAULT_PHOTOS_THUMB_SIZE;
    }

    public function getDisplayButtonArea()
    {
        if ($this->hasData('button_area')) {
            return (bool)$this->getData('button_area');
        }

        return false;
    }

    public function getButtonsPosition()
    {
        if ($this->hasData('button_position')) {
            return $this->getData('button_position');
        }

        return parent::getButtonsPosition();
    }

    public function getDisplayUploadButton()
    {
        if (!Mage::helper('mpgallery/photo')->canUpload()) {
            return false;
        }

        if ($this->hasData('upload_button')) {
            return (bool)$this->getData('upload_button');
        }

        return false;
    }

    public function getLimitPerPage()
    {
        if ($this->hasData('photo_per_page')) {
            return (int)$this->getData('photo_per_page');
        }

        return self::DEFAULT_PHOTOS_PER_PAGE;
    }

    public function getScrollToPhotos()
    {
        return false !== strpos(Mage::helper('core/url')->getCurrentUrl(), $this->getPageVarName());
    }

    public function getSortBy()
    {
        if ($this->hasData('photo_sort_by')) {
            return $this->getData('photo_sort_by');
        }

        return self::DEFAULT_PHOTOS_SORT_BY;
    }

    public function getSortDirection()
    {
        if ($this->hasData('photo_sort_dir')) {
            return $this->getData('photo_sort_dir');
        }

        return self::DEFAULT_PHOTOS_SORT_BY;
    }

    public function getDisplayRate()
    {
        if ($this->hasData('rate')) {
            return $this->getData('rate');
        }

        return false;
    }

    public function getImagesJson()
    {
        if (!$this->hasData('images_json')) {
            if ($this->_configHelper->isLightboxPagePhotos()) {
                $photos = $this->getCollection();
            } else {
                $photos = $this->getCollection()->getAllOrderedIds();
            }

            $photosImgs = array();
            foreach ($photos as $photo) {
                if (!is_object($photo)) {
                    $photo = Mage::getModel('mpgallery/photo')->load((int)$photo);
                }

                $photosImgs[] = array(
                    'url'   => $this->getImage($photo, 'image', $this->getPhotoSize())->__toString(),
                    'title' => $this->stripTags($photo->getName(), null, true),
                    'id'    => $photo->getUrlKey()
                );
            }

            $this->setData('images_json', Zend_Json::encode($photosImgs));
        }

        return $this->_getData('images_json');
    }

    public function getPhotoAvgRate($photo)
    {
        return Mage::getModel('mpgallery/review')->getPhotoAverageRate($photo->getId());
    }

    public function getUploadUrl()
    {
        return Mage::helper('mpgallery/url')->getGalleryUrl(array(
            'upload'   => 'photo',
            'album_id' => $this->getAlbumId(),
            'back'     => base64_encode(Mage::helper('core/url')->getCurrentUrl())
        ));
    }

    protected function _toHtml()
    {
        if ($this->isEnabled()
            && (($this->getCollection() && $this->getCollection()->getSize() > 0) || $this->getDisplayUploadButton())
        ) {
            return Mage_Core_Block_Template::_toHtml();
        }

        return '';
    }
}