import json
import os
import requests
import subprocess

# Sends requests to RocketShipIt
class RocketShipIt:
    def __init__(self, carrier='', api_key='', endpoint=''):
        self.carrier = carrier
        self.api_key = api_key
        self.endpoint = endpoint
        self.bin_name = 'RocketShipIt'

        # Directory that holds the RocketShipIt binary
        current_dir = os.path.dirname(os.path.realpath(__file__))
        self.bin_dir = current_dir + '/../../'

    # Make local request if RocketShipIt binary is available
    # else make HTTP API request.
    def request(self, carrier, action, params):
        self.carrier = carrier
        if os.path.isfile(self.bin_dir + self.bin_name):
            return self.local_request(action, params)
       
        return self.http_request(params, action)

    # Send JSON to local RocketShipIt binary app through
    # stdin/stdout
    def local_request(self, action, params):
        data = {
            'Carrier': self.carrier,
            'Action': action,
            'Params': params
        }
        popen = subprocess.Popen('./'+self.bin_name,
                                stdout=subprocess.PIPE,
                                stdin=subprocess.PIPE,
                                cwd=self.bin_dir)
        s = popen.communicate(json.dumps(data))

        return json.loads(s[0])
        
    # Send JSON over HTTP to RocketShipIt API
    def http_request(self, params, action):
        data = {
            'Carrier': self.carrier,
            'Action': action,
            'Params': params
        }
        headers = {'x-api-key': self.api_key}
        resp = requests.post(self.endpoint, json=data, headers=headers)

        return resp.json()
