<?php

/**

 * Magento

 *

 * NOTICE OF LICENSE

 *

 * This source file is subject to the Open Software License (OSL 3.0)

 * that is bundled with this package in the file LICENSE.txt.

 * It is also available through the world-wide-web at this URL:

 * http://opensource.org/licenses/osl-3.0.php

 * If you did not receive a copy of the license and are unable to

 * obtain it through the world-wide-web, please send an email

 * to license@magentocommerce.com so we can send you a copy immediately.

 *

 * DISCLAIMER

 *

 * Do not edit or add to this file if you wish to upgrade Magento to newer

 * versions in the future. If you wish to customize Magento for your

 * needs please refer to http://www.magentocommerce.com for more information.

 *

 * @category    Mage

 * @package     Mage_Catalog

 * @copyright   Copyright (c) 2014 Magento Inc. (http://www.magentocommerce.com)

 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)

 */





/**

 * Product options abstract type block

 *

 * @category   Mage

 * @package    Mage_Catalog

 * @author     Magento Core Team <core@magentocommerce.com>

 */

abstract class Mage_Catalog_Block_Product_View_Options_Abstract extends Mage_Core_Block_Template

{

    /**

     * Product object

     *

     * @var Mage_Catalog_Model_Product

     */

    protected $_product;



    /**

     * Product option object

     *

     * @var Mage_Catalog_Model_Product_Option

     */

    protected $_option;



    /**

     * Set Product object

     *

     * @param Mage_Catalog_Model_Product $product

     * @return Mage_Catalog_Block_Product_View_Options_Abstract

     */

    public function setProduct(Mage_Catalog_Model_Product $product = null)

    {

        $this->_product = $product;

        return $this;

    }



    /**

     * Retrieve Product object

     *

     * @return Mage_Catalog_Model_Product

     */

    public function getProduct()

    {

        return $this->_product;

    }



    /**

     * Set option

     *

     * @param Mage_Catalog_Model_Product_Option $option

     * @return Mage_Catalog_Block_Product_View_Options_Abstract

     */

    public function setOption(Mage_Catalog_Model_Product_Option $option)

    {

        $this->_option = $option;

        return $this;

    }



    /**

     * Get option

     *

     * @return Mage_Catalog_Model_Product_Option

     */

    public function getOption()

    {

        return $this->_option;

    }



    public function getFormatedPrice()

    {

        if ($option = $this->getOption()) {

            return $this->_formatPrice(array(

                'is_percent'    => ($option->getPriceType() == 'percent'),

                'pricing_value' => $option->getPrice($option->getPriceType() == 'percent')

            ));

        }

        return '';

    }



    /**

     * Return formated price

     *

     * @param array $value

     * @return string

     */

    protected function _formatPrice($value, $flag=true)

    {

        if ($value['pricing_value'] == 0) {

            return Mage::app()->getLocale()->currency(Mage::app()->getStore()->getCurrentCurrencyCode())->getSymbol().$this->getProduct()->getFinalPrice();

        }



        $taxHelper = Mage::helper('tax');

        $store = $this->getProduct()->getStore();



        $sign = '';

        if ($value['pricing_value'] < 0) {

            $sign = '';

            $value['pricing_value'] = 0 - $value['pricing_value'];

        }



        $priceStr = $sign;

        $_priceInclTax = $this->getPrice($value['pricing_value'], true)+$this->getProduct()->getFinalPrice();

$_priceExclTax = $this->getPrice($value['pricing_value'])+$this->getProduct()->getFinalPrice();



        if ($taxHelper->displayPriceIncludingTax()) {

            $priceStr .= $this->helper('core')->currencyByStore($_priceInclTax, $store, true, $flag);

        } elseif ($taxHelper->displayPriceExcludingTax()) {

            $priceStr .= $this->helper('core')->currencyByStore($_priceExclTax, $store, true, $flag);

        } elseif ($taxHelper->displayBothPrices()) {

            $priceStr .= $this->helper('core')->currencyByStore($_priceExclTax, $store, true, $flag);

            if ($_priceInclTax != $_priceExclTax) {

                $priceStr .= ' ('.$sign.$this->helper('core')

                    ->currencyByStore($_priceInclTax, $store, true, $flag).' '.$this->__('Incl. Tax').')';

            }

        }



        if ($flag) {

            $priceStr = '<span class="price-notice">'.$priceStr.'</span>';

        }



        return $priceStr;

    }



    /**

     * Get price with including/excluding tax

     *

     * @param decimal $price

     * @param bool $includingTax

     * @return decimal

     */

    public function getPrice($price, $includingTax = null)

    {

        if (!is_null($includingTax)) {

            $price = Mage::helper('tax')->getPrice($this->getProduct(), $price, true);

        } else {

            $price = Mage::helper('tax')->getPrice($this->getProduct(), $price);

        }

        return $price;

    }



    /**

     * Returns price converted to current currency rate

     *

     * @param float $price

     * @return float

     */

    public function getCurrencyPrice($price)

    {

        $store = $this->getProduct()->getStore();

        return $this->helper('core')->currencyByStore($price, $store, false);

    }

}

